from datetime import datetime, timedelta, timezone

from .user import User
from ..database import db
from secrets import compare_digest
from flaschengeist import logger


class Session(db.Model):
    """Model for a Session

    Args:
        expires: Is a Datetime from current Time.
        user:   Is an User.
        token:  String to verify access later.
    """

    __tablename__ = "session"
    id = db.Column(db.Integer, primary_key=True)
    user_id = db.Column(db.Integer, db.ForeignKey("user.id"))
    user: User = db.relationship("User", back_populates="sessions")

    expires: datetime = db.Column(db.DateTime)
    token: str = db.Column(db.String(32), unique=True)
    lifetime: int = db.Column(db.Integer)
    browser: str = db.Column(db.String(30))
    platform: str = db.Column(db.String(30))

    def refresh(self):
        """Update the Timestamp

        Update the Timestamp to the current Time.
        """
        logger.debug("update timestamp from session with token {{ {} }}".format(self))
        self.expires = datetime.utcnow() + timedelta(seconds=self.lifetime)

    def serialize(self):
        """Create Dic to dump in JSON

        Returns:
            A Dic with static Attributes.
        """
        return {
            "token": self.token,
            "expires": self.expires.replace(tzinfo=timezone.utc),
            "lifetime": self.lifetime,
            "user": self.user,
            "browser": self.browser,
            "platform": self.platform,
        }

    def __eq__(self, token):
        return compare_digest(self.token, token)
